@@ -14,6 +14,7 @@
 package cli
 
 import (
+	"context"
 	"encoding/csv"
 	"encoding/json"
 	"fmt"
@@ -21,24 +22,26 @@ import (
 	"math/rand"
 	"net"
 	"os"
+	"os/signal"
 	"runtime"
 	"strconv"
 	"strings"
 	"sync"
+	"syscall"
 	"time"
 
 	"github.com/zmap/zcrypto/x509"
 
 	"github.com/hashicorp/go-version"
 	"github.com/liip/sheriff"
-	"github.com/miekg/dns"
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
+	"github.com/zmap/dns"
 
-	"github.com/zmap/zdns/src/cli/iohandlers"
-	blacklist "github.com/zmap/zdns/src/internal/safeblacklist"
-	"github.com/zmap/zdns/src/internal/util"
-	"github.com/zmap/zdns/src/zdns"
+	"github.com/zmap/zdns/v2/src/cli/iohandlers"
+	blacklist "github.com/zmap/zdns/v2/src/internal/safeblacklist"
+	"github.com/zmap/zdns/v2/src/internal/util"
+	"github.com/zmap/zdns/v2/src/zdns"
 )
 
 type routineMetadata struct {
@@ -344,7 +347,9 @@ func populateIPTransportMode(gc *CLIConf, config *zdns.ResolverConfig) (*zdns.Re
 	// check OS' default resolver(s) to determine if we support IPv4 or IPv6
 	ipv4NSStrings, ipv6NSStrings, err = zdns.GetDNSServers(config.DNSConfigFilePath)
 	if err != nil {
-		log.Fatal("unable to parse resolvers file, please use '--name-servers': ", err)
+		log.Fatalf("ZDNS is unable to parse resolvers file. ZDNS only supports IPv4 and IPv6 addresses with an optional port, "+
+			" either 111.222.333.444:9953 or [1111:2222::3333]:9953. "+
+			"Please either modify your %s file or use '--name-servers'. Error: %v", config.DNSConfigFilePath, err)
 	}
 	if len(ipv4NSStrings) == 0 && len(ipv6NSStrings) == 0 {
 		return nil, errors.New("no nameservers found with OS defaults. Please specify desired nameservers with --name-servers")
@@ -480,6 +485,10 @@ func populateLocalAddresses(gc *CLIConf, config *zdns.ResolverConfig) (*zdns.Res
 }
 
 func Run(gc CLIConf) {
+	// Create a context that is cancelled on SIGINT or SIGTERM.
+	ctx, cancel := context.WithCancel(context.Background())
+	defer cancel()
+
 	gc = *populateCLIConfig(&gc)
 	resolverConfig := populateResolverConfig(&gc)
 	// Log any information about the resolver configuration, according to log level
@@ -500,10 +509,21 @@ func Run(gc CLIConf) {
 	//	- process until inChan closes, then wg.done()
 	// Once we processing threads have all finished, wait until the
 	// output and metadata threads have completed
-	inChan := make(chan string)
-	outChan := make(chan string)
-	metaChan := make(chan routineMetadata, gc.Threads)
-	statusChan := make(chan zdns.Status)
+	inChan := make(chan string)                        // input handler feeds inChan with input
+	outChan := make(chan string)                       // lookup workers write to outChan, output handler reads from outChan
+	metaChan := make(chan routineMetadata, gc.Threads) // lookup workers write to metaChan, metadata is collected at end of scan
+	statusChan := make(chan zdns.Status)               // lookup workers write to status chan after each lookup, status handler reads from statusChan
+	statusAbortChan := make(chan struct{})             // used by this thread to signal to the status handler that we've aborted
+
+	sigs := make(chan os.Signal, 1)
+	signal.Notify(sigs, syscall.SIGINT, syscall.SIGTERM)
+
+	go func() {
+		<-sigs                        // SIGINT or SIGTERM received
+		cancel()                      // signal to all goroutines to clean up
+		statusAbortChan <- struct{}{} // signal to status handler that we've aborted so it will print a suitable message
+		// to the user vs. the 'Scan Complete' it normally prints
+	}()
 	var routineWG sync.WaitGroup
 
 	inHandler := gc.InputHandler
@@ -523,7 +543,7 @@ func Run(gc CLIConf) {
 
 	// Use handlers to populate the input and output/results channel
 	go func() {
-		if inErr := inHandler.FeedChannel(inChan, &routineWG); inErr != nil {
+		if inErr := inHandler.FeedChannel(ctx, inChan, &routineWG); inErr != nil {
 			log.Fatal(fmt.Sprintf("could not feed input channel: %v", inErr))
 		}
 	}()
@@ -537,7 +557,7 @@ func Run(gc CLIConf) {
 
 	if !gc.QuietStatusUpdates {
 		go func() {
-			if statusErr := statusHandler.LogPeriodicUpdates(statusChan, &routineWG); statusErr != nil {
+			if statusErr := statusHandler.LogPeriodicUpdates(statusChan, statusAbortChan, &routineWG); statusErr != nil {
 				log.Fatal(fmt.Sprintf("could not log periodic status updates: %v", statusErr))
 			}
 		}()
@@ -552,7 +572,7 @@ func Run(gc CLIConf) {
 	for i := 0; i < gc.Threads; i++ {
 		i := i
 		go func(threadID int) {
-			initWorkerErr := doLookupWorker(&gc, resolverConfig, inChan, outChan, metaChan, statusChan, &lookupWG)
+			initWorkerErr := doLookupWorker(ctx, &gc, resolverConfig, inChan, outChan, metaChan, statusChan, &lookupWG)
 			if initWorkerErr != nil {
 				log.Fatalf("could not start lookup worker #%d: %v", i, initWorkerErr)
 			}
@@ -562,6 +582,7 @@ func Run(gc CLIConf) {
 	close(outChan)
 	close(metaChan)
 	close(statusChan)
+	close(statusAbortChan)
 	routineWG.Wait()
 	if gc.MetadataFilePath != "" {
 		// we're done processing data. aggregate all the data from individual routines
@@ -608,25 +629,27 @@ func Run(gc CLIConf) {
 }
 
 // doLookupWorker is a single worker thread that processes lookups from the input channel. It calls wg.Done when it is finished.
-func doLookupWorker(gc *CLIConf, rc *zdns.ResolverConfig, inputChan <-chan string, outputChan chan<- string, metaChan chan<- routineMetadata, statusChan chan<- zdns.Status, wg *sync.WaitGroup) error {
+func doLookupWorker(ctx context.Context, gc *CLIConf, rc *zdns.ResolverConfig, inputChan <-chan string, outputChan chan<- string, metaChan chan<- routineMetadata, statusChan chan<- zdns.Status, wg *sync.WaitGroup) error {
 	defer wg.Done()
 	resolver, err := zdns.InitResolver(rc)
 	if err != nil {
 		return fmt.Errorf("could not init resolver: %w", err)
 	}
+	defer resolver.Close() // close the resolver, freeing up resources
 	var metadata routineMetadata
 	metadata.Status = make(map[zdns.Status]int)
 
 	for line := range inputChan {
-		handleWorkerInput(gc, rc, line, resolver, &metadata, outputChan, statusChan)
+		if util.HasCtxExpired(ctx) {
+			break
+		}
+		handleWorkerInput(ctx, gc, rc, line, resolver, &metadata, outputChan, statusChan)
 	}
-	// close the resolver, freeing up resources
-	resolver.Close()
 	metaChan <- metadata
 	return nil
 }
 
-func handleWorkerInput(gc *CLIConf, rc *zdns.ResolverConfig, line string, resolver *zdns.Resolver, metadata *routineMetadata, outputChan chan<- string, statusChan chan<- zdns.Status) {
+func handleWorkerInput(ctx context.Context, gc *CLIConf, rc *zdns.ResolverConfig, line string, resolver *zdns.Resolver, metadata *routineMetadata, outputChan chan<- string, statusChan chan<- zdns.Status) {
 	// we'll process each module sequentially, parallelism is per-domain
 	res := zdns.Result{Results: make(map[string]zdns.SingleModuleResult, len(gc.ActiveModules))}
 	// get the fields that won't change for each lookup module
@@ -683,7 +706,7 @@ func handleWorkerInput(gc *CLIConf, rc *zdns.ResolverConfig, line string, resolv
 		res.Class = dns.Class(gc.Class).String()
 
 		startTime := time.Now()
-		innerRes, trace, status, err = module.Lookup(resolver, lookupName, nameServer)
+		innerRes, trace, status, err = module.Lookup(ctx, resolver, lookupName, nameServer)
 
 		lookupRes := zdns.SingleModuleResult{
 			Timestamp: time.Now().Format(gc.TimeFormat),
