@@ -22,6 +22,7 @@ import (
 	"reflect"
 	"regexp"
 	"sort"
+	"strings"
 	"testing"
 	"time"
 
@@ -29,9 +30,9 @@ import (
 
 	"github.com/stretchr/testify/assert"
 
-	"github.com/miekg/dns"
+	"github.com/zmap/dns"
 
-	"github.com/zmap/zdns/src/internal/util"
+	"github.com/zmap/zdns/v2/src/internal/util"
 )
 
 type nameAndIP struct {
@@ -649,7 +650,7 @@ func TestOneA(t *testing.T) {
 		Protocol:    "",
 		Flags:       DNSFlags{},
 	}
-	res, _, _, _ := resolver.DoTargetedLookup("example.com", ns1, false, true, false)
+	res, _, _, _ := resolver.DoTargetedLookup(context.Background(), "example.com", ns1, false, true, false)
 	verifyResult(t, *res, []string{"192.0.2.1"}, nil)
 }
 
@@ -684,7 +685,7 @@ func TestTwoA(t *testing.T) {
 		Protocol:    "",
 		Flags:       DNSFlags{},
 	}
-	res, _, _, _ := resolver.DoTargetedLookup(domain1, ns1, false, true, false)
+	res, _, _, _ := resolver.DoTargetedLookup(context.Background(), domain1, ns1, false, true, false)
 	verifyResult(t, *res, []string{"192.0.2.1", "192.0.2.2"}, nil)
 }
 
@@ -720,7 +721,7 @@ func TestQuadAWithoutFlag(t *testing.T) {
 		Flags:       DNSFlags{},
 	}
 
-	res, _, _, _ := resolver.DoTargetedLookup(domain1, ns1, false, true, false)
+	res, _, _, _ := resolver.DoTargetedLookup(context.Background(), domain1, ns1, false, true, false)
 	verifyResult(t, *res, []string{"192.0.2.1"}, nil)
 }
 
@@ -749,7 +750,7 @@ func TestOnlyQuadA(t *testing.T) {
 		Flags:       DNSFlags{},
 	}
 
-	res, _, _, _ := resolver.DoTargetedLookup(domain1, ns1, false, false, true)
+	res, _, _, _ := resolver.DoTargetedLookup(context.Background(), domain1, ns1, false, false, true)
 	assert.NotNil(t, res)
 	verifyResult(t, *res, nil, []string{"2001:db8::1"})
 }
@@ -785,7 +786,7 @@ func TestAandQuadA(t *testing.T) {
 		Protocol:    "",
 		Flags:       DNSFlags{},
 	}
-	res, _, _, _ := resolver.DoTargetedLookup(domain1, ns1, false, true, true)
+	res, _, _, _ := resolver.DoTargetedLookup(context.Background(), domain1, ns1, false, true, true)
 	assert.NotNil(t, res)
 	verifyResult(t, *res, []string{"192.0.2.1"}, []string{"2001:db8::1"})
 }
@@ -821,7 +822,7 @@ func TestTwoQuadA(t *testing.T) {
 		Protocol:    "",
 		Flags:       DNSFlags{},
 	}
-	res, _, _, _ := resolver.DoTargetedLookup("example.com", ns1, false, false, true)
+	res, _, _, _ := resolver.DoTargetedLookup(context.Background(), "example.com", ns1, false, false, true)
 	assert.NotNil(t, res)
 	verifyResult(t, *res, nil, []string{"2001:db8::1", "2001:db8::2"})
 }
@@ -845,7 +846,7 @@ func TestNoResults(t *testing.T) {
 		Protocol:    "",
 		Flags:       DNSFlags{},
 	}
-	res, _, _, _ := resolver.DoTargetedLookup("example.com", ns1, false, true, false)
+	res, _, _, _ := resolver.DoTargetedLookup(context.Background(), "example.com", ns1, false, true, false)
 	verifyResult(t, *res, nil, nil)
 }
 
@@ -880,7 +881,7 @@ func TestQuadAWithCname(t *testing.T) {
 		Protocol:    "",
 		Flags:       DNSFlags{},
 	}
-	res, _, _, _ := resolver.DoTargetedLookup("cname.example.com", ns1, false, false, true)
+	res, _, _, _ := resolver.DoTargetedLookup(context.Background(), "cname.example.com", ns1, false, false, true)
 	verifyResult(t, *res, nil, []string{"2001:db8::3"})
 }
 
@@ -909,7 +910,7 @@ func TestUnexpectedMxOnly(t *testing.T) {
 		Flags:       DNSFlags{},
 	}
 
-	res, _, status, _ := resolver.DoTargetedLookup("example.com", ns1, false, true, true)
+	res, _, status, _ := resolver.DoTargetedLookup(context.Background(), "example.com", ns1, false, true, true)
 
 	if status != StatusNoError {
 		t.Errorf("Expected no error, got %v", status)
@@ -957,7 +958,7 @@ func TestMxAndAdditionals(t *testing.T) {
 		Flags:       DNSFlags{},
 	}
 
-	res, _, _, _ := resolver.DoTargetedLookup("example.com", ns1, false, true, true)
+	res, _, _, _ := resolver.DoTargetedLookup(context.Background(), "example.com", ns1, false, true, true)
 	verifyResult(t, *res, []string{"192.0.2.3"}, []string{"2001:db8::4"})
 }
 
@@ -985,7 +986,7 @@ func TestMismatchIpType(t *testing.T) {
 		Flags:       DNSFlags{},
 	}
 
-	res, _, status, _ := resolver.DoTargetedLookup("example.com", ns1, false, false, true)
+	res, _, status, _ := resolver.DoTargetedLookup(context.Background(), "example.com", ns1, false, false, true)
 
 	if status != StatusNoError {
 		t.Errorf("Expected no error, got %v", status)
@@ -1033,11 +1034,11 @@ func TestEmptyNonTerminal(t *testing.T) {
 		Flags:       DNSFlags{},
 	}
 	// Verify leaf returns correctly
-	res, _, _, _ := resolver.DoTargetedLookup("leaf.intermediate.example.com", ns1, false, true, false)
+	res, _, _, _ := resolver.DoTargetedLookup(context.Background(), "leaf.intermediate.example.com", ns1, false, true, false)
 	verifyResult(t, *res, []string{"192.0.2.3"}, nil)
 
 	// Verify empty non-terminal returns no answer
-	res, _, _, _ = resolver.DoTargetedLookup("intermediate.example.com", ns1, false, true, true)
+	res, _, _, _ = resolver.DoTargetedLookup(context.Background(), "intermediate.example.com", ns1, false, true, true)
 	verifyResult(t, *res, nil, nil)
 }
 
@@ -1048,7 +1049,7 @@ func TestNXDomain(t *testing.T) {
 	resolver, err := InitResolver(config)
 	require.NoError(t, err)
 	ns1 := &config.ExternalNameServersV4[0]
-	res, _, status, _ := resolver.DoTargetedLookup("nonexistent.example.com", ns1, false, true, true)
+	res, _, status, _ := resolver.DoTargetedLookup(context.Background(), "nonexistent.example.com", ns1, false, true, true)
 	if status != StatusNXDomain {
 		t.Errorf("Expected StatusNXDomain status, got %v", status)
 	} else if res != nil {
@@ -1148,7 +1149,7 @@ func TestAandQuadADedup(t *testing.T) {
 		Flags:       DNSFlags{},
 	}
 
-	res, _, _, _ := resolver.DoTargetedLookup(domain1, ns1, false, true, true)
+	res, _, _, _ := resolver.DoTargetedLookup(context.Background(), domain1, ns1, false, true, true)
 	assert.NotNil(t, res)
 	verifyResult(t, *res, []string{"192.0.2.1"}, []string{"2001:db8::3"})
 }
@@ -1168,7 +1169,7 @@ func TestServFail(t *testing.T) {
 	name := "example.com"
 	protocolStatus[domainNS1] = StatusServFail
 
-	res, _, finalStatus, _ := resolver.DoTargetedLookup(name, ns1, false, true, true)
+	res, _, finalStatus, _ := resolver.DoTargetedLookup(context.Background(), name, ns1, false, true, true)
 
 	if finalStatus != protocolStatus[domainNS1] {
 		t.Errorf("Expected %v status, got %v", protocolStatus, finalStatus)
@@ -1230,7 +1231,7 @@ func TestNsAInAdditional(t *testing.T) {
 		IPv4Addresses: []string{"192.0.2.3"},
 		IPv6Addresses: nil,
 	}
-	res, _, _, _ := resolver.DoNSLookup("example.com", ns1, false, true, false)
+	res, _, _, _ := resolver.DoNSLookup(context.Background(), "example.com", ns1, false, true, false)
 	verifyNsResult(t, res.Servers, expectedServersMap)
 }
 
@@ -1291,7 +1292,7 @@ func TestTwoNSInAdditional(t *testing.T) {
 		IPv4Addresses: []string{"192.0.2.4"},
 		IPv6Addresses: nil,
 	}
-	res, _, _, _ := resolver.DoNSLookup("example.com", ns1, false, true, false)
+	res, _, _, _ := resolver.DoNSLookup(context.Background(), "example.com", ns1, false, true, false)
 	verifyNsResult(t, res.Servers, expectedServersMap)
 }
 
@@ -1340,7 +1341,7 @@ func TestAandQuadAInAdditional(t *testing.T) {
 		IPv4Addresses: []string{"192.0.2.3"},
 		IPv6Addresses: []string{"2001:db8::4"},
 	}
-	res, _, _, _ := resolver.DoNSLookup("example.com", ns1, false, true, true)
+	res, _, _, _ := resolver.DoNSLookup(context.Background(), "example.com", ns1, false, true, true)
 	verifyNsResult(t, res.Servers, expectedServersMap)
 }
 
@@ -1389,7 +1390,7 @@ func TestNsMismatchIpType(t *testing.T) {
 		IPv4Addresses: nil,
 		IPv6Addresses: nil,
 	}
-	res, _, _, _ := resolver.DoNSLookup("example.com", ns1, false, true, true)
+	res, _, _, _ := resolver.DoNSLookup(context.Background(), "example.com", ns1, false, true, true)
 	verifyNsResult(t, res.Servers, expectedServersMap)
 }
 
@@ -1450,7 +1451,7 @@ func TestAandQuadALookup(t *testing.T) {
 		IPv4Addresses: []string{"192.0.2.3"},
 		IPv6Addresses: []string{"2001:db8::4"},
 	}
-	res, _, _, _ := resolver.DoNSLookup("example.com", ns1, false, true, true)
+	res, _, _, _ := resolver.DoNSLookup(context.Background(), "example.com", ns1, false, true, true)
 	verifyNsResult(t, res.Servers, expectedServersMap)
 }
 
@@ -1461,7 +1462,7 @@ func TestNsNXDomain(t *testing.T) {
 
 	ns1 := &config.ExternalNameServersV4[0]
 
-	_, _, status, _ := resolver.DoNSLookup("nonexistentexample.com", ns1, false, true, true)
+	_, _, status, _ := resolver.DoNSLookup(context.Background(), "nonexistentexample.com", ns1, false, true, true)
 
 	assert.Equal(t, StatusNXDomain, status)
 }
@@ -1478,7 +1479,7 @@ func TestNsServFail(t *testing.T) {
 	mockResults[domainNS1] = SingleQueryResult{}
 	protocolStatus[domainNS1] = StatusServFail
 
-	res, _, status, _ := resolver.DoNSLookup("example.com", ns1, false, true, false)
+	res, _, status, _ := resolver.DoNSLookup(context.Background(), "example.com", ns1, false, true, false)
 
 	assert.Equal(t, status, protocolStatus[domainNS1])
 	assert.Empty(t, res.Servers)
@@ -1511,7 +1512,7 @@ func TestErrorInTargetedLookup(t *testing.T) {
 
 	protocolStatus[domainNS1] = StatusError
 
-	res, _, status, _ := resolver.DoNSLookup("example.com", ns1, false, true, false)
+	res, _, status, _ := resolver.DoNSLookup(context.Background(), "example.com", ns1, false, true, false)
 	assert.Empty(t, len(res.Servers), 0)
 	assert.Equal(t, status, protocolStatus[domainNS1])
 }
@@ -1691,7 +1692,7 @@ func TestAllNsLookupOneNsThreeLevels(t *testing.T) {
 		Name:  "example.com",
 	}
 
-	results, _, _, err := resolver.LookupAllNameserversIterative(&q, []NameServer{{DomainName: rootServer, IP: net.ParseIP(rootServerIP)}})
+	results, _, _, err := resolver.LookupAllNameserversIterative(context.Background(), &q, []NameServer{{DomainName: rootServer, IP: net.ParseIP(rootServerIP)}})
 	require.NoError(t, err)
 	verifyCombinedResult(t, results.LayeredResponses, expectedRes)
 }
@@ -1917,7 +1918,7 @@ func TestAllNsLookupErrorInOne(t *testing.T) {
 		Name:  "example.com",
 	}
 
-	results, _, _, err := resolver.LookupAllNameserversIterative(&q, []NameServer{{DomainName: rootServer, IP: net.ParseIP(rootServerIP)}})
+	results, _, _, err := resolver.LookupAllNameserversIterative(context.Background(), &q, []NameServer{{DomainName: rootServer, IP: net.ParseIP(rootServerIP)}})
 	require.NoError(t, err)
 	verifyCombinedResult(t, results.LayeredResponses, expectedRes)
 }
@@ -1935,7 +1936,7 @@ func TestAllNsLookupNXDomain(t *testing.T) {
 		Name:  "example.com",
 	}
 
-	res, _, status, err := resolver.LookupAllNameserversIterative(&q, []NameServer{*ns1})
+	res, _, status, err := resolver.LookupAllNameserversIterative(context.Background(), &q, []NameServer{*ns1})
 
 	expectedResponse := map[string][]ExtendedResult{
 		".": {{Status: StatusNXDomain, Type: "NS"}},
@@ -1972,6 +1973,109 @@ func TestInvalidInputsLookup(t *testing.T) {
 	})
 }
 
+func TestGetDNSServersFromReader(t *testing.T) {
+	tests := []struct {
+		name     string
+		input    string
+		wantIPv4 []string
+		wantIPv6 []string
+		wantErr  bool
+	}{
+		{
+			name:     "Valid IPv4 address with default port",
+			input:    "nameserver 1.2.3.4",
+			wantIPv4: []string{"1.2.3.4:53"},
+			wantIPv6: nil,
+			wantErr:  false,
+		},
+		{
+			name:     "Valid IPv6 address with default port",
+			input:    "nameserver 2001:db8::1",
+			wantIPv4: nil,
+			wantIPv6: []string{"[2001:db8::1]:53"},
+			wantErr:  false,
+		},
+		{
+			name:     "Valid IPv6 compressed address with default port",
+			input:    "nameserver ::1",
+			wantIPv4: nil,
+			wantIPv6: []string{"[::1]:53"},
+			wantErr:  false,
+		},
+		{
+			name:     "Valid IPv6 partially-compressed address with default port",
+			input:    "nameserver 2001:db8:0:0:0::1",
+			wantIPv4: nil,
+			wantIPv6: []string{"[2001:db8:0:0:0::1]:53"},
+			wantErr:  false,
+		},
+		{
+			name:     "Valid IPv4 with custom port",
+			input:    "nameserver 1.2.3.4:35",
+			wantIPv4: []string{"1.2.3.4:35"},
+			wantIPv6: nil,
+			wantErr:  false,
+		},
+		{
+			name:     "Valid IPv6 with custom port",
+			input:    "nameserver [2001:db8::1]:35",
+			wantIPv4: nil,
+			wantIPv6: []string{"[2001:db8::1]:35"},
+			wantErr:  false,
+		},
+		{
+			name:     "Invalid IPv4 address",
+			input:    "nameserver 1.2.3",
+			wantIPv4: nil,
+			wantIPv6: nil,
+			wantErr:  true,
+		},
+		{
+			name:     "IPv6 link-local address (should error)",
+			input:    "nameserver fe80::1%eth0",
+			wantIPv4: nil,
+			wantIPv6: nil,
+			wantErr:  true,
+		},
+		{
+			name:     "Invalid format - interface specified on IPv4",
+			input:    "nameserver 111.222.333.444:9953%ifname",
+			wantIPv4: nil,
+			wantIPv6: nil,
+			wantErr:  true,
+		},
+		{
+			name:     "Invalid format - interface specified on IPv6",
+			input:    "nameserver [2001:db8::1]]:9953%ifname",
+			wantIPv4: nil,
+			wantIPv6: nil,
+			wantErr:  true,
+		},
+		{
+			name:     "Invalid format - domain specified",
+			input:    "nameserver 111.222.333.444:9953#example.com",
+			wantIPv4: nil,
+			wantIPv6: nil,
+			wantErr:  true,
+		},
+	}
+
+	for _, test := range tests {
+		t.Run(test.name, func(t *testing.T) {
+			ipv4, ipv6, err := getDNSServersFromReader(strings.NewReader(test.input))
+			if (err != nil) != test.wantErr {
+				t.Errorf("getDNSServersFromReader() received error = %v, wantErr %v", err, test.wantErr)
+			}
+			if fmt.Sprintf("%v", ipv4) != fmt.Sprintf("%v", test.wantIPv4) {
+				t.Errorf("getDNSServersFromReader() received ipv4 = %v, want %v", ipv4, test.wantIPv4)
+			}
+			if fmt.Sprintf("%v", ipv6) != fmt.Sprintf("%v", test.wantIPv6) {
+				t.Errorf("getDNSServersFromReader() received ipv6 = %v, want %v", ipv6, test.wantIPv6)
+			}
+		})
+	}
+}
+
 func verifyNsResult(t *testing.T, servers []NSRecord, expectedServersMap map[string]IPResult) {
 	serversLength := len(servers)
 	expectedServersLength := len(expectedServersMap)
