@@ -26,13 +26,13 @@ import (
 
 	"github.com/pkg/errors"
 
-	"github.com/miekg/dns"
 	log "github.com/sirupsen/logrus"
+	"github.com/zmap/dns"
 	"github.com/zmap/zcrypto/tls"
 	"github.com/zmap/zgrab2/lib/http"
 
-	blacklist "github.com/zmap/zdns/src/internal/safeblacklist"
-	"github.com/zmap/zdns/src/internal/util"
+	blacklist "github.com/zmap/zdns/v2/src/internal/safeblacklist"
+	"github.com/zmap/zdns/v2/src/internal/util"
 )
 
 const (
@@ -224,9 +224,13 @@ func NewResolverConfig() *ResolverConfig {
 		LookupClient: LookupClient{},
 		Cache:        c,
 
-		Blacklist:    blacklist.New(),
-		LocalAddrsV4: []net.IP{},
-		LocalAddrsV6: []net.IP{},
+		Blacklist:             blacklist.New(),
+		LocalAddrsV4:          []net.IP{},
+		LocalAddrsV6:          []net.IP{},
+		ExternalNameServersV4: DefaultExternalResolversV4,
+		ExternalNameServersV6: DefaultExternalResolversV6,
+		RootNameServersV4:     RootServersV4,
+		RootNameServersV6:     RootServersV6,
 
 		TransportMode:         defaultTransportMode,
 		IPVersionMode:         defaultIPVersionMode,
@@ -617,6 +621,8 @@ func (r *Resolver) ExternalLookup(ctx context.Context, q *Question, dstServer *N
 	if r.isClosed {
 		log.Fatal("resolver has been closed, cannot perform lookup")
 	}
+	ctx, cancelFn := context.WithTimeout(ctx, r.timeout)
+	defer cancelFn()
 	// If dstServer is not provided, AND we're in HTTPS/TLS/TCP mode, AND we have a pre-existing external name server, use it
 	if dstServer == nil && r.lastUsedExternalNameServer == nil {
 		dstServer = r.randomExternalNameServer()
@@ -645,6 +651,8 @@ func (r *Resolver) IterativeLookup(ctx context.Context, q *Question) (*SingleQue
 	if r.isClosed {
 		log.Fatal("resolver has been closed, cannot perform lookup")
 	}
+	ctx, cancelFn := context.WithTimeout(ctx, r.timeout)
+	defer cancelFn()
 	return r.lookupClient.DoDstServersLookup(ctx, r, *q, r.rootNameServers, true)
 }
 
